/****************************************************************************
** $Id: qg_layerwidget.cpp,v 1.9 2003/09/23 14:03:04 andrew Exp $
**
** Copyright (C) 2001-2003 RibbonSoft. All rights reserved.
**
** This file is part of the qcadlib Library project.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding valid qcadlib Professional Edition licenses may use 
** this file in accordance with the qcadlib Commercial License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.ribbonsoft.com for further details.
**
** Contact info@ribbonsoft.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "qg_layerwidget.h"

#include <qtoolbutton.h>
#include <qcursor.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpopupmenu.h>
#include <qtooltip.h>

#include "xpm/visiblelayer.xpm"
#include "xpm/hiddenlayer.xpm"
#include "xpm/layeradd.xpm"
#include "xpm/layerremove.xpm"
#include "xpm/layeredit.xpm"



/**
 * Constructor.
 */
QG_LayerWidget::QG_LayerWidget(QG_ActionHandler* ah, QWidget* parent,
                               const char* name, WFlags f)
        : QWidget(parent, name, f),
        pxmVisible(visiblelayer_xpm),
        pxmHidden(hiddenlayer_xpm),
        pxmAdd(layeradd_xpm),
        pxmRemove(layerremove_xpm),
        pxmEdit(layeredit_xpm),
        pxmDefreezeAll(visiblelayer_xpm),
pxmFreezeAll(hiddenlayer_xpm) {

    actionHandler = ah;
    layerList = NULL;
    showByBlock = false;

    listBox = new QListBox(this, "layerbox");
    listBox->setDragSelect(false);
    listBox->setMultiSelection(false);
    listBox->setSmoothScrolling(true);
	listBox->setFocusPolicy(QWidget::NoFocus);
	listBox->setMinimumHeight(140);

    QVBoxLayout* lay = new QVBoxLayout(this, 0, -1, "lay");

    /*QLabel* caption = new QLabel(tr("Layer List"), this, "lLayers");
    caption->setAlignment(Qt::AlignCenter);
    caption->setPaletteBackgroundColor(black);
    caption->setPaletteForegroundColor(white);
	*/

    QHBoxLayout* layButtons = new QHBoxLayout(NULL, 0, -1, "layButtons");
    QToolButton* but;
    // show all layer:
    but = new QToolButton(this);
    but->setPixmap(pxmDefreezeAll);
    but->setMinimumSize(QSize(22,22));
    QToolTip::add(but, tr("Show all layers"));
    connect(but, SIGNAL(clicked()),
            actionHandler, SLOT(slotLayersDefreezeAll()));
    layButtons->addWidget(but);
    // hide all layer:
    but = new QToolButton(this);
    but->setPixmap(pxmFreezeAll);
    but->setMinimumSize(QSize(22,22));
    QToolTip::add(but, tr("Hide all layers"));
    connect(but, SIGNAL(clicked()),
            actionHandler, SLOT(slotLayersFreezeAll()));
    layButtons->addWidget(but);
    // add layer:
    but = new QToolButton(this);
    but->setPixmap(pxmAdd);
    but->setMinimumSize(QSize(22,22));
    QToolTip::add(but, tr("Add a layer"));
    connect(but, SIGNAL(clicked()),
            actionHandler, SLOT(slotLayersAdd()));
    layButtons->addWidget(but);
    // remove layer:
    but = new QToolButton(this);
    but->setPixmap(pxmRemove);
    but->setMinimumSize(QSize(22,22));
    QToolTip::add(but, tr("Remove the current layer"));
    connect(but, SIGNAL(clicked()),
            actionHandler, SLOT(slotLayersRemove()));
    layButtons->addWidget(but);
    // rename layer:
    but = new QToolButton(this);
    but->setPixmap(pxmEdit);
    but->setMinimumSize(QSize(22,22));
    QToolTip::add(but, tr("Modify layer attributes / rename"));
    connect(but, SIGNAL(clicked()),
            actionHandler, SLOT(slotLayersEdit()));
    layButtons->addWidget(but);

    //lay->addWidget(caption);
    lay->addLayout(layButtons);
    lay->addWidget(listBox);

    connect(listBox, SIGNAL(highlighted(const QString&)),
            this, SLOT(slotActivated(const QString&)));

    connect(listBox, SIGNAL(doubleClicked(QListBoxItem*)),
            actionHandler, SLOT(slotLayersToggleView()));
}



/**
 * Destructor
 */
QG_LayerWidget::~QG_LayerWidget() {
    delete listBox;
    //delete pxmVisible;
    //delete pxmHidden;
}



/**
 * Sets the layerlist this layer widget should show.
 *
 * @param showByBlock true: show the layer with the name "ByBlock" if
 *                    it exists.
 *                    false: don't show special layer "ByBlock"
 */
void QG_LayerWidget::setLayerList(RS_LayerList* layerList, bool showByBlock) {
    this->layerList = layerList;
    this->showByBlock = showByBlock;
    update();
}



/**
 * Updates the layer box from the layers in the graphic.
 */
void QG_LayerWidget::update() {
    RS_DEBUG->print("QG_LayerWidget::update() begin");

    int yPos = listBox->contentsY();

    RS_Layer* activeLayer;
    if (layerList!=NULL) {
        activeLayer = layerList->getActive();
    } else {
        activeLayer = NULL;
    }

    listBox->clear();

    if (layerList==NULL) {
        RS_DEBUG->print("QG_LayerWidget::update() abort");
        return;
    }

    for (uint i=0; i<layerList->count(); ++i) {
        RS_Layer* layer = layerList->at(i);

        // hide layer "ByBlock"?
        if (showByBlock || layer->getName()!="ByBlock") {
            if (!layer->isFrozen()) {
                listBox->insertItem(pxmVisible, layer->getName());
            } else {
                listBox->insertItem(pxmHidden, layer->getName());
            }
        }
    }

    listBox->sort();

    highlightLayer(activeLayer);
    listBox->setContentsPos(0, yPos);

    RS_DEBUG->print("QG_LayerWidget::update() end");
}


/**
 * Highlights (activates) the given layer and makes it 
 * the active layer in the layerlist.
 */
void QG_LayerWidget::highlightLayer(RS_Layer* layer) {
    RS_DEBUG->print("QG_LayerWidget::highlightLayer() begin");

    if (layer==NULL || layerList==NULL) {
        RS_DEBUG->print("QG_LayerWidget::highlightLayer() abort");
        return;
    }

    layerList->activate(layer);
    QString name = layer->getName();

    for (int i=0; i<(int)listBox->count(); ++i) {
        if (listBox->text(i)==name) {
            listBox->setCurrentItem(i);
            break;
        }
    }

    RS_DEBUG->print("QG_LayerWidget::highlightLayer() end");
}



/**
 * Called when the user activates (highlights) a layer.
 */
void QG_LayerWidget::slotActivated(const QString& layerName) {
    RS_DEBUG->print("QG_LayerWidget::slotActivated(): %s", layerName.latin1());

    if (layerList==NULL) {
        return;
    }

    layerList->activate(layerName);
}


/**
 * Shows a context menu for the layer widget. Launched with a right click.
 */
void QG_LayerWidget::contextMenuEvent(QContextMenuEvent *e) {

    if (actionHandler!=NULL) {
        QPopupMenu* contextMenu = new QPopupMenu(this);
        QLabel* caption = new QLabel(tr("Layer Menu"), this);
        caption->setPaletteBackgroundColor(black);
        caption->setPaletteForegroundColor(white);
        caption->setAlignment( Qt::AlignCenter );
        contextMenu->insertItem( caption );
        contextMenu->insertItem( tr("&Defreeze all Layers"), actionHandler,
                                 SLOT(slotLayersDefreezeAll()), 0);
        contextMenu->insertItem( tr("&Freeze all Layers"), actionHandler,
                                 SLOT(slotLayersFreezeAll()), 0);
        contextMenu->insertItem( tr("&Add Layer"), actionHandler,
                                 SLOT(slotLayersAdd()), 0);
        contextMenu->insertItem( tr("&Remove Layer"), actionHandler,
                                 SLOT(slotLayersRemove()), 0);
        contextMenu->insertItem( tr("&Edit Layer"), actionHandler,
                                 SLOT(slotLayersEdit()), 0);
        contextMenu->insertItem( tr("&Toggle Visibility"), actionHandler,
                                 SLOT(slotLayersToggleView()), 0);
        contextMenu->exec(QCursor::pos());
        delete contextMenu;
    }

    e->accept();
}


/**
 * Escape releases focus.
 */
void QG_LayerWidget::keyPressEvent(QKeyEvent* e) {
	//std::cout << "QG_LayerWidget::keyPressEvent\n";
	switch (e->key()) {
	
	case Qt::Key_Escape:
		//std::cout << "layerwidget: escape\n";
		emit escape();
		break;
		
	default:
		QWidget::keyPressEvent(e);
		break;
	}
}

// EOF
